@@ -3,6 +3,7 @@
 # SPDX-License-Identifier: MPL-2.0
 
 import re
+import os
 import traceback
 import urllib.parse as ulp
 import panel as pn
@@ -25,7 +26,7 @@ If the simulation host is set to "localhost", a local server will be started aut
     schematic = param.String()
     database_url = param.String(label="Database URL")
     spice = param.String(precedence=-1)
-    netlist = param.Dict(precedence=-1)
+    vectors = param.List(precedence=-1)
     
     simulator = param.Selector(objects={"NgSpice": simserver.Ngspice, "Xyce": simserver.Xyce})
     host = param.String(default="localhost")
@@ -47,7 +48,9 @@ If the simulation host is set to "localhost", a local server will be started aut
             # actually use localhost because Jupyter is token authenticated
             purl = ulp.urlparse(url)
             if purl.hostname == "localhost" and purl.path.startswith("/couchdb"):
-                netloc = f"admin:admin@localhost:5984"
+                pw = os.environ.get("COUCHDB_ADMIN_PASSWORD", "admin")
+                port = os.environ.get("COUCHDB_LISTEN_PORT", 5984)
+                netloc = f"admin:{pw}@localhost:{port}"
                 path = purl.path[8:]
                 url = purl._replace(path=path, netloc=netloc).geturl()
                 self.database_url = url
@@ -58,8 +61,10 @@ If the simulation host is set to "localhost", a local server will be started aut
                 async for schem in it:
                     if url != self.database_url or name != self.schematic:
                         break
-                    self.netlist = schem
                     self.spice = netlist.spice_netlist(name, schem, self.extra_spice)
+                    if self.simulator == simserver.Ngspice:
+                        self.vectors = netlist.ngspice_vectors(name, schem)
+                        print(self.vectors)
                     self.error = None
         except Exception as e:
             self.error = e
@@ -85,8 +90,9 @@ If the simulation host is set to "localhost", a local server will be started aut
 
 
 class Simulation(param.Parameterized):
-    vectors = param.List(precedence=0.5)
+    vectors = param.ListSelector(default=[], precedence=0.5)
     rerun_on_change = param.Boolean(True, precedence=0.5)
+    back_annotate = param.Boolean(False, precedence=0.5)
 
     def cmd(self, fs):
         raise NotImplementedError()
@@ -101,6 +107,8 @@ Perform a non-linear, time-domain simulation"""
     maximum_timestep = param.Number(1e-5)
     start_time = param.Number(0)
     stop_time = param.Number(1e-3)
+
+    rerun_on_change = param.Boolean(False, precedence=0.5)
     
     def cmd(self, fs):
         return fs.commands.tran(self.maximum_timestep, self.stop_time, self.start_time, self.vectors)
@@ -141,6 +149,8 @@ class OpSimulation(Simulation):
 Find the DC operating point, treating capacitances as open circuits and inductors as shorts
 """
 
+    back_annotate = param.Boolean(True, precedence=0.5)
+
     def cmd(self, fs):
         return fs.commands.op(self.vectors)
 
@@ -212,7 +222,7 @@ class SimTabs(param.Parameterized):
             AcSimulation(name="AC simulation"),
             DcSimulation(name="DC sweep"),
             NoiseSimulation(name="Noise simulation"),
-            FftSimulation(name="FFT simulation"),
+            #FftSimulation(name="FFT simulation"),
         ])
 
     def cb(self, _, e):
@@ -245,13 +255,16 @@ class Results(param.Parameterized):
         super().__init__()
         self.terminal = pn.widgets.Terminal(height=300, sizing_mode='stretch_width')
 
-    async def simulate(self, _=None):
+    async def simulate(self, *, save=False, database_url=None, name=None):
         try:
             for v in self.data.values():
                 v.clear()
             res = await self.cmd()
             newkey = lambda k: self.param.trigger('data')
             await simserver.stream(res, self.data, newkey, self.terminal)
+            if save and database_url and name:
+                async with netlist.SchematicService(database_url) as service:
+                    await service.save_simulation(name, self.data)
         except Exception as e:
             self.error = e
             traceback.print_exc()
@@ -325,8 +338,8 @@ class Simulator(param.Parameterized):
         )
         self.brch = BroadcastChannel(name="probe", sizing_mode='fixed')
         self.brch.param.watch(self._probe, ['value'], onlychanged=False)
-        self.conf.param.watch(self._run, ['spice'])
-        self.param.watch(self._run, ['stage'], onlychanged=False)
+        self.conf.param.watch(self._run_change, ['spice'])
+        self.param.watch(self._run_btn, ['stage'], onlychanged=False)
         self.tabs.param.watch(self._cmd, ['sim'])
         self._cmd(None)
 
@@ -337,12 +350,28 @@ class Simulator(param.Parameterized):
         self.res.data = {}
         self.res.cmd = simcmd
         self.res.plotcmd = self.tabs.sim.plotcmd
+        self.tabs.sim.param.vectors.objects = self.conf.vectors
     
-    async def _run(self, e):
+    async def _run_change(self, e):
+        self.tabs.sim.param.vectors.objects = self.conf.vectors
         if (self.res.cmd
         and self.stage == "res"
         and self.tabs.sim.rerun_on_change):
-            await self.res.simulate()
+            await self.res.simulate(
+                save=self.tabs.sim.back_annotate,
+                database_url=self.conf.database_url,
+                name=self.conf.schematic,
+            )
+
+    async def _run_btn(self, e):
+        if (self.res.cmd
+        and self.stage == "res"):
+            await self.res.simulate(
+                save=self.tabs.sim.back_annotate,
+                database_url=self.conf.database_url,
+                name=self.conf.schematic,
+            )
+
 
     def _probe(self, e):
         key = e.obj.value
